/** @file wlan_exp_node_ap.h
 *  @brief Access Point WLAN Experiment
 *
 *  This contains code for the 802.11 Access Point's WLAN experiment interface.
 *
 *  @copyright Copyright 2013-2019, Mango Communications. All rights reserved.
 *          Distributed under the Mango Communications Reference Design License
 *              See LICENSE.txt included in the design archive or
 *              at http://mangocomm.com/802.11/license
 *
 *  This file is part of the Mango 802.11 Reference Design (https://mangocomm.com/802.11)
 */



/***************************** Include Files *********************************/
#include "wlan_mac_high_sw_config.h"

#include "wlan_exp_common.h"

struct cmd_resp_hdr_t;
struct eth_tx_queue_buffer_t;


/*************************** Constant Definitions ****************************/
#ifndef WLAN_EXP_NODE_AP_H_
#define WLAN_EXP_NODE_AP_H_



// ****************************************************************************
// Define Node Commands
//
// NOTE:  All Command IDs (CMDID_*) must be a 24 bit unique number
//

//-----------------------------------------------
// WLAN Exp Node AP Commands
//
#define CMDID_NODE_AP_CONFIG                                         0x100000
#define CMDID_NODE_AP_SET_AUTHENTICATION_ADDR_FILTER                 0x100001

#define CMD_PARAM_NODE_AP_CONFIG_FLAG_DTIM_MULTICAST_BUFFER          0x00000001

#define CMD_PARAM_AP_ASSOCIATE_FLAG_DISABLE_INACTIVITY_TIMEOUT       0x00000001
#define CMD_PARAM_AP_ASSOCIATE_FLAG_HT_CAPABLE_STA                   0x00000004

// ****************************************************************************
// Define Node AP Parameters
//   - NOTE:  To add another parameter, add the define before "NODE_MAX_PARAMETER"
//     and then change the value of "NODE_MAX_PARAMETER" to be the largest value
//     in the list so it is easy to iterate over all parameters
//


/*********************** Global Structure Definitions ************************/



/*************************** Function Prototypes *****************************/

#if WLAN_SW_CONFIG_ENABLE_WLAN_EXP
int process_wlan_exp_app_cmd(struct cmd_resp_hdr_t* cmd_hdr, struct eth_tx_queue_buffer_t* eth_tx_queue_buffer);
#endif //WLAN_SW_CONFIG_ENABLE_WLAN_EXP

#endif /* WLAN_EXP_NODE_H_ */
