/** @file wlan_platform_high.h
 *  @brief High Platform Defines for 802.11 Ref design
 *
 *  @copyright Copyright 2014-2019, Mango Communications. All rights reserved.
 *          Distributed under the Mango Communications Reference Design License
 *                See LICENSE.txt included in the design archive or
 *                at http://mangocomm.com/802.11/license
 *
 *  This file is part of the Mango 802.11 Reference Design (https://mangocomm.com/802.11)
 */

#ifndef WLAN_PLATFORM_HIGH_H_
#define WLAN_PLATFORM_HIGH_H_

#include "xil_types.h"
#include "xstatus.h"
#include "wlan_common_types.h"
#include "wlan_high_types.h"
#include "wlan_platform_intc.h"
#include "wlan_mac_high_sw_config.h"

// Forward declarations
struct eth_tx_queue_buffer_t;

#if WLAN_SW_CONFIG_ENABLE_WLAN_EXP
struct cmd_resp_hdr_t;
#endif

//---------------------------------------
// Platform information struct
typedef struct platform_high_dev_info_t{
	u32		aux_bram_baseaddr;
	u32		aux_bram_size;
	u32		dram_baseaddr;
	u32		dram_size;
	u32		cdma_dev_id;
	u32 	mailbox_int_id;
	u32		timer_dur_us;
} platform_high_dev_info_t;

typedef enum userio_disp_status_t{
	USERIO_DISP_STATUS_IDENTIFY     		= 0,
	USERIO_DISP_STATUS_APPLICATION_ROLE     = 1,
	USERIO_DISP_STATUS_MEMBER_LIST_UPDATE   = 2,
	USERIO_DISP_STATUS_WLAN_EXP_CONFIGURE   = 3,
	USERIO_DISP_STATUS_CPU_ERROR    		= 255
} userio_disp_high_status_t;

//---------------------------------------
// Public functions that WLAN MAC High Framework directly calls
void wlan_platform_cpu_high_init();
platform_high_dev_info_t wlan_platform_high_get_dev_info();
int wlan_platform_high_init();
void wlan_platform_free_queue_entry_notify();
void wlan_platform_high_poll();
int wlan_platform_high_copy_info(u8* dest);


// User IO functions
void wlan_platform_high_userio_disp_status(userio_disp_high_status_t status, int status_param);

// SW interrupt functions
void wlan_platform_assert_sw_intr(u32 intr_mask);
void wlan_platform_clear_sw_intr(u32 intr_mask);
void wlan_platform_set_sw_intr_mask(u32 intr_mask);
void wlan_platform_clear_sw_intr_mask(u32 intr_mask);

#if WLAN_SW_CONFIG_ENABLE_WLAN_EXP
int wlan_platform_wlan_exp_process_node_cmd(struct cmd_resp_hdr_t* cmd_hdr, struct eth_tx_queue_buffer_t* eth_tx_queue_buffer, u8* cmd_processed);
#endif

#if WLAN_SW_CONFIG_ENABLE_ETH_BRIDGE
int wlan_platform_portal_eth_send( struct eth_tx_queue_buffer_t* eth_tx_queue_buffer );
int wlan_platform_portal_eth_get_mtu();
#endif //WLAN_SW_CONFIG_ENABLE_ETH_BRIDGE

#if WLAN_SW_CONFIG_ENABLE_WLAN_EXP
int wlan_platform_wlan_exp_eth_send( struct eth_tx_queue_buffer_t* eth_tx_queue_buffer );
int wlan_platform_wlan_exp_eth_get_mtu();
#endif //WLAN_SW_CONFIG_ENABLE_WLAN_EXP

#endif /* WLAN_PLATFORM_HIGH_H_ */
